#ifndef BABELTRACE2_TRACE_IR_STREAM_CLASS_CONST_H
#define BABELTRACE2_TRACE_IR_STREAM_CLASS_CONST_H

/*
 * Copyright (c) 2010-2019 EfficiOS Inc. and Linux Foundation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __BT_IN_BABELTRACE_H
# error "Please include <babeltrace2/babeltrace.h> instead."
#endif

#include <stdint.h>

#include <babeltrace2/types.h>

#ifdef __cplusplus
extern "C" {
#endif

extern const bt_value *bt_stream_class_borrow_user_attributes_const(
		const bt_stream_class *stream_class);

extern const bt_trace_class *bt_stream_class_borrow_trace_class_const(
		const bt_stream_class *stream_class);

extern const char *bt_stream_class_get_name(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_assigns_automatic_event_class_id(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_assigns_automatic_stream_id(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_supports_packets(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_packets_have_beginning_default_clock_snapshot(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_packets_have_end_default_clock_snapshot(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_supports_discarded_events(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_supports_discarded_packets(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_discarded_events_have_default_clock_snapshots(
		const bt_stream_class *stream_class);

extern bt_bool bt_stream_class_discarded_packets_have_default_clock_snapshots(
		const bt_stream_class *stream_class);

extern uint64_t bt_stream_class_get_id(
		const bt_stream_class *stream_class);

extern const bt_field_class *
bt_stream_class_borrow_packet_context_field_class_const(
		const bt_stream_class *stream_class);

extern const bt_field_class *
bt_stream_class_borrow_event_common_context_field_class_const(
		const bt_stream_class *stream_class);

extern uint64_t bt_stream_class_get_event_class_count(
		const bt_stream_class *stream_class);

extern const bt_event_class *
bt_stream_class_borrow_event_class_by_index_const(
		const bt_stream_class *stream_class, uint64_t index);

extern const bt_event_class *
bt_stream_class_borrow_event_class_by_id_const(
		const bt_stream_class *stream_class, uint64_t id);

extern const bt_clock_class *
bt_stream_class_borrow_default_clock_class_const(
		const bt_stream_class *stream_class);

extern void bt_stream_class_get_ref(const bt_stream_class *stream_class);

extern void bt_stream_class_put_ref(const bt_stream_class *stream_class);

#define BT_STREAM_CLASS_PUT_REF_AND_RESET(_var)		\
	do {						\
		bt_stream_class_put_ref(_var);		\
		(_var) = NULL;				\
	} while (0)

#define BT_STREAM_CLASS_MOVE_REF(_var_dst, _var_src)	\
	do {						\
		bt_stream_class_put_ref(_var_dst);	\
		(_var_dst) = (_var_src);		\
		(_var_src) = NULL;			\
	} while (0)

#ifdef __cplusplus
}
#endif

#endif /* BABELTRACE2_TRACE_IR_STREAM_CLASS_CONST_H */
